unit cvWebComponents;

interface
{ $Define TrialVersion}
uses
  Windows, Messages, SysUtils, Classes, WebComp, MidItems, WebForm, SiteComp;

type
  IClientScript = interface
  ['{7DC7F32B-F12F-4322-9EC2-C3EEFA3A098B}']
    function ClientScript: string;
  end;

  IClientValidation = interface
  ['{3608EC72-9EDF-490C-BED5-325A3B4C8F4D}']
    function SupportClientValidation: Boolean;
    function ClientValidationProc: string;
    function ClientValidationScript: string;
    function InputFieldToValidate: TObject;
  end;

  TcvRegularExpression = string;
  TcvCompareOperator = (co_Equal, co_NotEqual, co_GreaterThan, co_GreaterThanEqual,
         co_LessThan, co_LessThanEqual);
  TcvDataType = (dt_Number, dt_String, dt_Date);

  TcvAdapterDisplayField = class(TAdapterDisplayField)
  private
    FInvalidMsg: string;
    function CalcInputElementType(var AType: TInputFieldHTMLElementType): Boolean;
    function ValidationAttribute: string;
    function ValidationAttributeId: string;
  protected
    { IWebContent }
    function ImplContent(Options: TWebContentOptions; ParentLayout: TLayout): string; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property InvalidMsg: string read FInvalidMsg write FInvalidMsg;
  end;

  TcvCustomValidator = class(TWebContainedComponent, IClientScript, IClientValidation)
  private
    FFieldToValidate: TcvAdapterDisplayField;
    FClientValidation: Boolean;
    function GetClientValidation: Boolean;
    function GetClientValidationProc: string;
    function GetClientValidationVisibilityAttrib: string;
    function GetFullQualifiedFieldName(AField: TAdapterDisplayField): string;
    function GetParentForm(AComponent: TComponent): TComponent;
  protected
    { IClientScript }
    function ClientScript: string;
    function ImplClientScript: string; virtual;
    { IClientValidation }
    function SupportClientValidation: Boolean;
    function ClientValidationProc: string;
    function ClientValidationScript: string;
    function InputFieldToValidate: TObject;
    function ValidationExpression: string;
    function ImplSupportClientValidation: Boolean; virtual;
    function ImplClientValidationProc: string; virtual;
    function ImplClientValidationScript: string; virtual;
    function ImplValidationExpression: string; virtual;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property FieldToValidate: TcvAdapterDisplayField read FFieldToValidate write FFieldToValidate;
    property ClientValidation: Boolean read FClientValidation write FClientValidation default true;
  end;

  TcvRequiredFieldValidator = class(TcvCustomValidator)
  private
  protected
    function ImplValidationExpression: string; override;
  public
  published
  end;

  TcvCompareValidator = class(TcvCustomValidator)
  private
    FFieldToCompare: TcvAdapterDisplayField;
    FValueToCompare: string;
    FOperator: TcvCompareOperator;
    FDataType: TcvDataType;
    function OperationExpression: string;
  protected
    function ImplValidationExpression: string; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property FieldToCompare: TcvAdapterDisplayField read FFieldToCompare write FFieldToCompare;
    property ValueToCompare: string read FValueToCompare write FValueToCompare;
    property Operator: TcvCompareOperator read FOperator write FOperator default co_Equal;
    property DataType: TcvDataType read FDataType write FDataType default dt_String;
  end;

  TcvRangeValidator = class(TcvCustomValidator)
  private
    FMinimumValue,FMaximumValue: string;
    FDataType: TcvDataType;
  protected
    function ImplValidationExpression: string; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property MinimumValue: string read FMinimumValue write FMinimumValue;
    property MaximumValue: string read FMaximumValue write FMaximumValue;
    property DataType: TcvDataType read FDataType write FDataType default dt_Number;
  end;

  TcvRegularExpressionValidator = class(TcvCustomValidator)
  private
    FRegularExpression: TcvRegularExpression;
  protected
    function ImplValidationExpression: string; override;
    function NormalizedValidationExpression: string;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property RegularExpression: TcvRegularExpression read FRegularExpression write FRegularExpression;
  end;

  TcvAdapterForm = class(TAdapterForm, IClientScript)
  private
    { Private declarations }
    function GetHTMLFormOnSubmitProc: string;
    procedure InsertOnSubmitAttrib(var AContent: String);
    function IsCommonFunctionsScriptImplemented: Boolean;
    function IsNeededOnSubmitValidation: Boolean;
    function OnSubmitScript: string;
    function CompareNumbersScript: string;
    function CompareDatesScript: string;
    function DateRangeCheckerScript: string;
    function NumberRangeCheckerScript: string;
    function RegularExpressionCheckerScript: string;
    function StateCheckerScript: string;
{$ifdef TrialVersion}
    function TrialVersionTitle: string;
{$endif}
  protected
    { Protected declarations }
    { IWebComponentEditor }
    function ImplCanAddClass(AParent: TComponent; AClass: TClass): Boolean; override;
    { IWebContent }
    function ImplContent(Options: TWebContentOptions; ParentLayout: TLayout): string; override;
    { IClientScript }
    function ClientScript: string;
    function ImplClientScript: string; virtual;
    function ImplCommonFunctionsScript: string; virtual;
  public
    { Public declarations }
  published
    { Published declarations }
  end;


implementation

const
  cpt_Function = 'function';
  cpt_Return = 'return';
  cpt_True = 'true';
  cpt_False = 'false';
  cpt_Var = 'var';
  cpt_ValidationProc = 'Validate()';
  cpt_And = ' && ';
  cpt_Value = 'value';
  cpt_SelectedIndex = 'selectedIndex';
  cpt_Options = 'options';
  cpt_Text = 'text';
  cpt_Equal = ' == ';
  cpt_NotEqual = ' != ';
  cpt_GreaterThan = ' > ';
  cpt_GreaterThanEqual = ' >= ';
  cpt_LessThan = ' < ';
  cpt_LessThanEqual = ' <= ';
  cpt_EmptyString = '""';
  cpt_Name = 'name';
  cpt_CloseColumnTag = '</td>';
  cpt_Style = 'style';
  cpt_Visibility = 'display';
  cpt_ReturnValue = 'ReturnValue';
  cpt_NumberRangeChecker = 'NumberRangeChecker';
  cpt_DateRangeChecker = 'DateRangeChecker';
  cpt_RegularExpressionChecker = 'RegularExpressionChecker';
  cpt_CompareNumbers = 'CompareNumbers';
  cpt_CompareDates = 'CompareDates';
  cpt_FieldValue = 'FieldValue';
  cpt_MinValue = 'MinValue';
  cpt_MaxValue = 'MaxValue';
  cpt_StateChecker = 'StateChecker';
  cpt_FullQualifiedFieldName = 'FullQualifiedFieldName';
  cpt_IsNotNumber = 'isNaN';
  cpt_DateParse = 'Date.parse';

function LastPos(Substr: string; S: string): Integer;
var
 APos: Integer;
begin
 Result := Pos(Substr,S);
 APos := Result;
 while (APos > 0) do
  begin
   Delete(S,1,APos+Length(Substr)-1);
   APos := Pos(Substr,S);
   if APos > 0 then Result := Result + Length(Substr) + APos-1;
  end;
end;

function CalcDefaultAdapterInputElementType(AField: TComponent; const AAdapterMode: string): TAdapterInputHTMLElementType;
var
  Intf: IGetHTMLStyle;
  S: string;
  I: TAdapterInputHTMLElementType;
begin
  Result := htmliTextInput;
  if Supports(IUnknown(AField), IGetHTMLStyle, Intf) then
  begin
    S := Intf.GetInputStyle(AAdapterMode);
    for I := Low(TAdapterInputHTMLElementType) to High(TAdapterInputHTMLElementType) do
      if CompareText(S, AdapterInputHTMLElementTypeNames[I]) = 0 then
      begin
        Result := I;
        break
      end;
  end;
end;

function TcvAdapterDisplayField.CalcInputElementType(var AType: TInputFieldHTMLElementType): Boolean;
  function ConvertInputType(T: TAdapterInputHTMLElementType): TInputFieldHTMLElementType;
  begin
    case T of
      htmliTextInput: Result := iftTextInput;
      htmliPasswordInput:  Result := iftPasswordInput;
      htmliSelect:         Result := iftSelect;
      htmliSelectMultiple: Result := iftSelectMultiple;
      htmliRadio:          Result := iftRadio;
      htmliCheckBox:       Result := iftCheckBox;
      htmliTextArea:       Result := iftTextArea;
      htmliFile:            Result := iftFile;
    else
      Result := iftTextInput;
      Assert(False, 'unexpected type');
    end;
  end;
var
  Field: TComponent;
  AdapterInputType: TAdapterInputHTMLElementType;
begin
  if InputType = iftDefault then
  begin
    AType := iftTextInput;
    Result := True;
    Field := FindAssociatedField;
    if Field <> nil then
    begin
      AdapterInputType := CalcDefaultAdapterInputElementType(Field, GetAdapterModeOfAdapter);
      if AdapterInputType <> htmliNone then
      begin
        AType := ConvertInputType(AdapterInputType);
        Result := True;
      end
      else
        Result := False;
    end;
  end
  else
  begin
    AType := InputType;
    Result := True;
  end;
end;

constructor TcvAdapterDisplayField.Create(AOwner: TComponent);
begin
 Inherited;
 FInvalidMsg := '*';
end;

function TcvAdapterDisplayField.ImplContent(Options: TWebContentOptions; ParentLayout: TLayout): string;
var
 CloseColumnTagPos: Integer;
begin
 Result := Inherited ImplContent(Options, ParentLayout);
 CloseColumnTagPos := LastPos(cpt_CloseColumnTag,Result);
 if CloseColumnTagPos > 0 then
  CloseColumnTagPos := CloseColumnTagPos + Length(cpt_CloseColumnTag)
 else CloseColumnTagPos := Length(Result);
 Insert(ValidationAttribute,Result,CloseColumnTagPos);
end;

function TcvAdapterDisplayField.ValidationAttribute: string;
begin
 Result :=
  '<td><span id="' + ValidationAttributeId +
  '" style="color:Red;width:100%;display:none;">' + FInvalidMsg + '</span></td>';
end;

function TcvAdapterDisplayField.ValidationAttributeId: string;
const
 cpt_ValidationAttribute = 'ValidationAttribute';
begin
 Result := Self.Name + '_' + cpt_ValidationAttribute;
end;

constructor TcvCustomValidator.Create(AOwner: TComponent);
begin
 Inherited;
 FClientValidation := true;
 FFieldToValidate := nil;
end;

function TcvCustomValidator.GetClientValidation: Boolean;
begin
 Result := FClientValidation and
  Assigned(FFieldToValidate) and not (hoHideAlways in FFieldToValidate.HideOptions);
end;

function TcvCustomValidator.GetFullQualifiedFieldName(AField: TAdapterDisplayField): string;
var
 ParentForm: TComponent;
begin
 Result := AField.FieldName;
 ParentForm := GetParentForm(AField);
 if ParentForm <> nil then Result := ParentForm.Name + '.' + Result;
end;

function TcvCustomValidator.GetParentForm(AComponent: TComponent): TComponent;
var
 AParent: TComponent;
begin
 Result := nil;
 AParent := AComponent.GetParentComponent;
 while (AParent <> nil) and not (AParent is TAdapterForm) do
  AParent := AParent.GetParentComponent;
 if (AParent <> nil) and (AParent is TAdapterForm) then Result := AParent;
end;

function TcvCustomValidator.ClientScript: string;
begin
 Result := ImplClientScript;
end;

function TcvCustomValidator.ImplClientScript: string;
begin
 Result := ImplClientValidationScript;
end;

function TcvCustomValidator.SupportClientValidation: Boolean;
begin
 Result := ImplSupportClientValidation;
end;

function TcvCustomValidator.ClientValidationProc: string;
begin
 Result := ImplClientValidationProc;
end;

function TcvCustomValidator.ClientValidationScript: string;
begin
 Result := ImplClientValidationScript;
end;

function TcvCustomValidator.InputFieldToValidate: TObject;
begin
 Result := FFieldToValidate;
end;

function TcvCustomValidator.ValidationExpression: string;
begin
 Result := ImplValidationExpression;
end;

function TcvCustomValidator.ImplSupportClientValidation: Boolean;
begin
 Result := GetClientValidation;
end;

function TcvCustomValidator.ImplClientValidationProc: string;
begin
 Result := GetClientValidationProc;
end;

function TcvCustomValidator.ImplClientValidationScript: string;
begin
 Result := '';
 if GetClientValidation then
  Result :=
   cpt_Function + ' ' + GetClientValidationProc + ' {' + #13#10 +
   ' if (' + ValidationExpression + ') {' + #13#10 +
   '  ' + GetClientValidationVisibilityAttrib + ' = "none";' + #13#10 +
   '  ' + cpt_Return + ' ' + cpt_True + ';' + #13#10 +
   ' }' + #13#10 +
   ' else {' + #13#10 +
   '  ' + GetClientValidationVisibilityAttrib + ' = "inline";' + #13#10 +
   '  ' + cpt_Return + ' ' + cpt_False + ';' + #13#10 +
   ' }' + #13#10 +
   '}';
end;

function TcvCustomValidator.ImplValidationExpression: string;
begin
 Result := cpt_True;
end;

function TcvCustomValidator.GetClientValidationProc: string;
begin
 Result := Self.Name + '_' + cpt_ValidationProc;
end;

function TcvCustomValidator.GetClientValidationVisibilityAttrib: string;
begin
 Result :=
  FFieldToValidate.ValidationAttributeId + '.' + cpt_Style + '.' + cpt_Visibility;
end;

function TcvRequiredFieldValidator.ImplValidationExpression: string;
var
 InputType: TInputFieldHTMLElementType;
begin
 Result := cpt_True;
 if FFieldToValidate.CalcInputElementType(InputType) then
  case InputType of
   iftTextInput,
   iftTextArea,
   iftFile,
   iftPasswordInput  :
    Result :=
     GetFullQualifiedFieldName(FFieldToValidate) + '.' + cpt_Value +
     cpt_NotEqual + cpt_EmptyString;
   iftSelect,
   iftSelectMultiple :
    Result :=
     '(' + GetFullQualifiedFieldName(FFieldToValidate) + '.' + cpt_SelectedIndex + ' >= 0)' +
     cpt_And + '(' + GetFullQualifiedFieldName(FFieldToValidate) + '.' + cpt_Options +
     '[' + GetFullQualifiedFieldName(FFieldToValidate) + '.' + cpt_SelectedIndex + '].' +
     cpt_Text + cpt_NotEqual + cpt_EmptyString + ')';
   iftRadio,
   iftCheckBox       :
    Result := cpt_StateChecker + '(' + GetFullQualifiedFieldName(FFieldToValidate) + ')';
  end;
end;

constructor TcvCompareValidator.Create(AOwner: TComponent);
begin
 Inherited;
 FFieldToCompare := nil;
 FValueToCompare := '';
 FOperator := co_Equal;
 FDataType := dt_String;
end;

function TcvCompareValidator.OperationExpression: string;
begin
 Result := cpt_Equal;
 case FOperator of
  co_NotEqual        : Result := cpt_NotEqual;
  co_GreaterThan     : Result := cpt_GreaterThan;
  co_GreaterThanEqual: Result := cpt_GreaterThanEqual;
  co_LessThan        : Result := cpt_LessThan;
  co_LessThanEqual   : Result := cpt_LessThanEqual;
 end;
end;

function TcvCompareValidator.ImplValidationExpression: string;
var
 FieldToValidateInputType,FieldToCompareInputType: TInputFieldHTMLElementType;
 CompareExpression: string;
begin
 Result := cpt_True;
 if FFieldToValidate.CalcInputElementType(FieldToValidateInputType) and
    (FieldToValidateInputType in [iftTextInput,iftTextArea,iftFile,iftPasswordInput]) and
    (not Assigned(FFieldToCompare) or
     FFieldToCompare.CalcInputElementType(FieldToCompareInputType) and
     (FieldToCompareInputType in [iftTextInput,iftTextArea,iftFile,iftPasswordInput])) then
   begin
    if Assigned(FFieldToCompare) then
     CompareExpression := GetFullQualifiedFieldName(FFieldToCompare) + '.' + cpt_Value
    else CompareExpression := '"' + FValueToCompare + '"';
    case FDataType of
     dt_Number :
      Result :=
       cpt_CompareNumbers + '(' +
        GetFullQualifiedFieldName(FFieldToValidate) + '.' + cpt_Value + ',' +
        CompareExpression + ',"' + OperationExpression + '")';
     dt_String :
      Result :=
       GetFullQualifiedFieldName(FFieldToValidate) + '.' + cpt_Value +
       OperationExpression + CompareExpression;
     dt_Date   :
      Result :=
       cpt_CompareDates + '(' +
        GetFullQualifiedFieldName(FFieldToValidate) + '.' + cpt_Value + ',' +
        CompareExpression + ',"' + OperationExpression + '")';
    end;
   end;
end;

constructor TcvRangeValidator.Create(AOwner: TComponent);
begin
 Inherited;
 FMinimumValue := '0';
 FMaximumValue := '100';
 FDataType := dt_Number;
end;

function TcvRangeValidator.ImplValidationExpression: string;
var
 FieldToValidateInputType: TInputFieldHTMLElementType;
begin
 Result := cpt_True;
 if FFieldToValidate.CalcInputElementType(FieldToValidateInputType) and
    (FieldToValidateInputType in [iftTextInput,iftTextArea,iftFile,iftPasswordInput]) then
   case FDataType of
    dt_Number :
      Result :=
       cpt_NumberRangeChecker + '(' +
        GetFullQualifiedFieldName(FFieldToValidate) + '.' + cpt_Value + ',' +
        '"' + FMinimumValue + '",' + '"' + FMaximumValue + '")';
    dt_String :
      Result :=
       '(' + GetFullQualifiedFieldName(FFieldToValidate) + '.' + cpt_Value +
       cpt_GreaterThanEqual + '"' + FMinimumValue + '")' + cpt_And +
       '(' + GetFullQualifiedFieldName(FFieldToValidate) + '.' + cpt_Value +
       cpt_LessThanEqual + '"' + FMaximumValue + '")';
    dt_Date   :
      Result :=
       cpt_DateRangeChecker + '(' +
        GetFullQualifiedFieldName(FFieldToValidate) + '.' + cpt_Value + ',' +
        '"' + FMinimumValue + '",' + '"' + FMaximumValue + '")';
   end;
end;

constructor TcvRegularExpressionValidator.Create(AOwner: TComponent);
begin
 Inherited;
 FRegularExpression := '';
end;

function TcvRegularExpressionValidator.NormalizedValidationExpression: string;
var
 i: Integer;
 DoInsert: Boolean;
begin
 Result := FRegularExpression;
 DoInsert := false;
 i := Length(Result);
 while (i > 0) do
  begin
   if (Result[i] = '\') then DoInsert := not DoInsert
   else if DoInsert then
    begin
     Insert('\',Result,i+1);
     DoInsert := false;
    end;
   Dec(i);
  end;
end;

function TcvRegularExpressionValidator.ImplValidationExpression: string;
var
 FieldToValidateInputType: TInputFieldHTMLElementType;
begin
 Result := cpt_True;
 if FFieldToValidate.CalcInputElementType(FieldToValidateInputType) and
    (FieldToValidateInputType in [iftTextInput,iftTextArea,iftFile,iftPasswordInput]) then
   Result :=
     cpt_RegularExpressionChecker + '(' +
      GetFullQualifiedFieldName(FFieldToValidate) + '.' + cpt_Value + ',' +
      '"' + NormalizedValidationExpression + '")';
end;

function TcvAdapterForm.ImplCanAddClass(AParent: TComponent; AClass: TClass): Boolean;
begin
 Result :=
  Inherited ImplCanAddClass(AParent,AClass) or
  AClass.InheritsFrom(TcvCustomValidator);
end;

function TcvAdapterForm.ImplContent(Options: TWebContentOptions; ParentLayout: TLayout): string;
begin
 Result :=
{$ifdef TrialVersion}
  TrialVersionTitle + #13#10 +
{$endif}
  ClientScript +
  Inherited ImplContent(Options, ParentLayout);
 if IsNeededOnSubmitValidation then InsertOnSubmitAttrib(Result);
end;

function TcvAdapterForm.ClientScript: string;
begin
 Result := ImplClientScript;
end;

function TcvAdapterForm.ImplClientScript: string;
var
 Intf: IClientScript;
 i: Integer;
begin
 Result := OnSubmitScript;
 for i := 0 to WebFieldControls.Count - 1 do
  if Supports(IInterface(WebFieldControls[I]), IClientScript, Intf) then
   begin
    if Result <> '' then Result := Result + #13#10;
    Result := Result + Intf.ClientScript;
   end;
 if Result <> '' then Result :=
   '<script language="JavaScript1.2">' + #13#10 +
   '<!--' + #13#10 + Result + #13#10 + '-->' + #13#10 +
   '</script>' + #13#10;
end;

function TcvAdapterForm.GetHTMLFormOnSubmitProc: string;
const
 OnSubmitProc = 'OnSubmit()';
begin
 Result := Self.Name + '_' + OnSubmitProc;
end;

procedure TcvAdapterForm.InsertOnSubmitAttrib(var AContent: String);
const
 FormTag = '<form';
 CloseTag = '>';
 sOnSubmitAttr = 'onsubmit';
var
 Attrib: String;
 FormTagPos,i: Integer;
begin
 AddQuotedAttrib(Attrib, sOnSubmitAttr, cpt_Return + ' ' + GetHTMLFormOnSubmitProc);
 FormTagPos := Pos(FormTag,AContent);
 if FormTagPos > 0 then
  begin
   i := FormTagPos + Length(FormTag);
   while (i < Length(AContent)) and (AContent[i] <> CloseTag) do Inc(i);
   if AContent[i] = CloseTag then Insert(Attrib,AContent,i);
  end;
end;

function TcvAdapterForm.IsCommonFunctionsScriptImplemented: Boolean;
var
 AComp,AParent: TComponent;
 Intf: IGetWebComponentList;
 AComponentList: TWebComponentList;
 i: Integer;
begin
 Result := false;
 AParent := GetParentComponent;
 if (AParent <> nil) and
    (Supports(IInterface(AParent), IGetWebComponentList, Intf)) then
  begin
   AComponentList := TWebComponentList(Intf.GetComponentList);
   if AComponentList <> nil then
    with AComponentList do
     if Count > 0 then
      for i := 0 to Count-1 do
       begin
        AComp := WebComponents[i];
        if AComp is TcvAdapterForm then
         if AComp = Self then break
         else if TcvAdapterForm(AComp).IsNeededOnSubmitValidation then
          begin
           Result := true;
           break;
          end;
       end;
  end;
end;

function TcvAdapterForm.IsNeededOnSubmitValidation: Boolean;
var
 Intf: IClientValidation;
 i: Integer;
begin
 Result := false;
 for i := 0 to WebFieldControls.Count - 1 do
  if Supports(IInterface(WebFieldControls[I]), IClientValidation, Intf) and
     Intf.SupportClientValidation then
   begin
    Result := true;
    exit;
   end;
end;

function TcvAdapterForm.OnSubmitScript: string;
var
 ValidationGroups: TStrings;
 FieldToValidate: TObject;
 Intf: IClientValidation;
 ValidatorsScript,ValidatorProc: string;
 i,Idx: Integer;
 IsNeededScript: Boolean;

begin
 Result := '';
 IsNeededScript := false;
 ValidatorsScript := '';
 ValidationGroups := TStringList.Create;
 try
  for i := 0 to WebFieldControls.Count - 1 do
   if Supports(IInterface(WebFieldControls[I]), IClientValidation, Intf) and
      Intf.SupportClientValidation then
    begin
     ValidatorProc := Intf.ClientValidationProc;
     FieldToValidate := Intf.InputFieldToValidate;
     if (FieldToValidate <> nil) and (ValidatorProc <> '') then
      begin
       IsNeededScript := true;
       Idx := ValidationGroups.IndexOfObject(FieldToValidate);
       if Idx >= 0 then
        ValidationGroups[Idx] := ValidationGroups[Idx] + cpt_And + ValidatorProc
       else ValidationGroups.AddObject(ValidatorProc,FieldToValidate);
      end;
    end;
 if IsNeededScript then
  begin
   with ValidationGroups do
    for i := 0 to Count-1 do
     begin
      if ValidatorsScript <> '' then ValidatorsScript := ValidatorsScript + #13#10;
      ValidatorsScript := ValidatorsScript +
       ' if (!(' + ValidationGroups[i] + ')) ' + cpt_ReturnValue + ' = ' + cpt_False + ';';
     end;
   Result :=
    cpt_Function + ' ' + GetHTMLFormOnSubmitProc + ' {' + #13#10 +
    ' ' + cpt_Var + ' ' + cpt_ReturnValue + ' = ' + cpt_True + ';' + #13#10 +
    ValidatorsScript + #13#10 +
    ' ' + cpt_Return + ' ' + cpt_ReturnValue + ';' + #13#10 +
    '}';
   if not IsCommonFunctionsScriptImplemented then
    Result := ImplCommonFunctionsScript + #13#10 + Result;
  end;
 finally
  ValidationGroups.Free;
 end;
end;


function TcvAdapterForm.StateCheckerScript: string;
begin
 Result :=
  cpt_Function + ' ' +
  cpt_StateChecker + '(' + cpt_FullQualifiedFieldName + ') {' + #13#10 +
  ' var RetValue = true;' + #13#10 +
  ' for (var i = 0; i < ' + cpt_FullQualifiedFieldName + '.length; i++) {' + #13#10 +
  '  RetValue = false;' + #13#10 +
  '  if (' + cpt_FullQualifiedFieldName + '[i].checked) return true;' + #13#10 +
  ' }' + #13#10 +
  ' return RetValue;' + #13#10 +
  '}';
end;

{$ifdef TrialVersion}
function TcvAdapterForm.TrialVersionTitle: string;
begin
 Result :=
  '<p><font face="Arial" size="2" color="#FF0000">' +
  'You are using the cvWebComponents package 1.0 trial version!' + '</font></p>';
end;
{$endif}

function TcvAdapterForm.NumberRangeCheckerScript: string;
begin
 Result :=
  cpt_Function + ' ' +
  cpt_NumberRangeChecker + '(FieldValue,MinValue,MaxValue) {' + #13#10 +
  ' var nValue = parseFloat(FieldValue);' + #13#10 +
  ' var nMinValue = parseFloat(MinValue);' + #13#10 +
  ' var nMaxValue = parseFloat(MaxValue);' + #13#10 +
  ' return (!isNaN(nValue) && !isNaN(nMinValue) && !isNaN(nMaxValue) &&' +
     ' (nValue >= nMinValue) && (nValue <= nMaxValue))' + #13#10 +
  '}';
end;

function TcvAdapterForm.CompareNumbersScript: string;
begin
 Result :=
  cpt_Function + ' ' +
  cpt_CompareNumbers + '(FieldValue,CompareValue,Operator) {' + #13#10 +
  ' var nValue = parseFloat(FieldValue);' + #13#10 +
  ' var nCompareValue = parseFloat(CompareValue);' + #13#10 +
  ' return (!isNaN(nValue) && !isNaN(nCompareValue) &&' +
     ' eval(nValue + Operator + nCompareValue))' + #13#10 +
  '}';
end;

function TcvAdapterForm.CompareDatesScript: string;
begin
 Result :=
  cpt_Function + ' ' +
  cpt_CompareDates + '(FieldValue,CompareValue,Operator) {' + #13#10 +
  ' var nValue = Date.parse(FieldValue);' + #13#10 +
  ' var nCompareValue = Date.parse(CompareValue);' + #13#10 +
  ' return (!isNaN(nValue) && !isNaN(nCompareValue) &&' +
     ' eval(nValue + Operator + nCompareValue))' + #13#10 +
  '}';
end;

function TcvAdapterForm.DateRangeCheckerScript: string;
begin
 Result :=
  cpt_Function + ' ' +
  cpt_DateRangeChecker + '(FieldValue,MinValue,MaxValue) {' + #13#10 +
  ' var nValue = Date.parse(FieldValue);' + #13#10 +
  ' var nMinValue = Date.parse(MinValue);' + #13#10 +
  ' var nMaxValue = Date.parse(MaxValue);' + #13#10 +
  ' return (!isNaN(nValue) && !isNaN(nMinValue) && !isNaN(nMaxValue) &&' +
     ' (nValue >= nMinValue) && (nValue <= nMaxValue))' + #13#10 +
  '}';
end;

function TcvAdapterForm.RegularExpressionCheckerScript: string;
begin
 Result :=
  cpt_Function + ' ' +
  cpt_RegularExpressionChecker + '(FieldValue,RegularExpession) {' + #13#10 +
  ' var re = new RegExp(RegularExpession);' + #13#10 +
  ' return re.test(FieldValue)' + #13#10 +
  '}';
end;

function TcvAdapterForm.ImplCommonFunctionsScript: string;
begin
 Result :=
  StateCheckerScript + #13#10 +
  NumberRangeCheckerScript + #13#10 +
  DateRangeCheckerScript + #13#10 +
  CompareNumbersScript + #13#10 +
  CompareDatesScript + #13#10 +
  RegularExpressionCheckerScript;
end;


end.
